@@ -37,6 +37,7 @@ class Restart(Exception):
     """
     Cause the debugger to restart for the target Python program.
     """
+
     pass
 
 
@@ -44,6 +45,7 @@ class ClientClose(Exception):
     """
     Cause the debugger to wait for a new client to connect.
     """
+
     pass
 
 
@@ -51,6 +53,7 @@ class DebugState(Enum):
     """
     Enumerates the three possible states of a debugging session.
     """
+
     NOT_STARTED = 0
     STARTING = 1
     STARTED = 2
@@ -60,7 +63,7 @@ def command_buffer(debugger):
     """
     Buffer input from a socket, yield complete debugger commands.
     """
-    remainder = b''
+    remainder = b""
     while True:
         new_buffer = debugger.client.recv(1024)
         if new_buffer:
@@ -76,16 +79,17 @@ def command_buffer(debugger):
             if terminator is None:
                 remainder = commands.pop()
             else:
-                remainder = b''
+                remainder = b""
             for command in commands:
-                command = command.decode('utf-8')
+                command = command.decode("utf-8")
                 command_data = json.loads(command)
                 logging.debug(command_data)
                 debugger.commands.put(command_data)
         else:
             # If recv() returns None, the socket is closed.
+            logger.debug("Closing debugger socket.")
             break
-    debugger.commands.put(('close', {}))
+    debugger.commands.put(("close", {}))
 
 
 class Debugger(bdb.Bdb):
@@ -93,7 +97,7 @@ class Debugger(bdb.Bdb):
     Instances of this class represent and drive the debugging process.
     """
 
-    ETX = b'\x03'  # End transmission token.
+    ETX = b"\x03"  # End transmission token.
 
     def __init__(self, socket, host, port, skip=None):
         super().__init__(skip=skip)
@@ -101,7 +105,7 @@ class Debugger(bdb.Bdb):
         self.socket = socket
         self.host = host
         self.port = port
-        self.mainpyfile = ''
+        self.mainpyfile = ""
         self.client = None
         self.command_thread = None
         self.commands = None
@@ -118,14 +122,14 @@ class Debugger(bdb.Bdb):
         Dumps data related to a referenced event to the socket.
         """
         try:
-            dumped = json.dumps((event, data)).encode('utf-8')
+            dumped = json.dumps((event, data)).encode("utf-8")
             logging.debug(dumped)
             self.client.sendall(dumped + Debugger.ETX)
         except OSError as e:
-            logger.debug('Debugger client error.')
+            logger.debug("Debugger client error.")
             logger.debug(e)
         except AttributeError as e:
-            logger.debug('Debugger client not connected to runner.')
+            logger.debug("Debugger client not connected to runner.")
             logger.debug(e)
 
     def output_stack(self):
@@ -138,9 +142,9 @@ class Debugger(bdb.Bdb):
         """
         str_index = 0
         sl = len(self.stack)  # Bound check for stack length.
-        if sl > 1 and self.stack[1][0].f_code.co_filename == '<string>':
+        if sl > 1 and self.stack[1][0].f_code.co_filename == "<string>":
             str_index = 2
-        elif sl > 3 and self.stack[3][0].f_code.co_filename == '<string>':
+        elif sl > 3 and self.stack[3][0].f_code.co_filename == "<string>":
             str_index = 4
         stack_data = []
         if str_index > 0:
@@ -148,24 +152,28 @@ class Debugger(bdb.Bdb):
                 frame_data = (
                     line_no,
                     {
-                        'filename': frame.f_code.co_filename,
-                        'locals': {k: repr(v) for k, v in
-                                   frame.f_locals.items()},
-                        'globals': {k: repr(v) for k, v in
-                                    frame.f_globals.items()},
-                        'builtins': {k: repr(v) for k, v in
-                                     frame.f_builtins.items()},
-                        'restricted': getattr(frame, 'f_restricted', ''),
-                        'lasti': repr(frame.f_lasti),
-                        'exc_type': repr(getattr(frame, 'f_exc_type', '')),
-                        'exc_value': repr(getattr(frame, 'f_exc_value', '')),
-                        'exc_traceback': repr(getattr(frame,
-                                                      'f_exc_traceback', '')),
-                        'current': frame is self.curframe,
-                    }
+                        "filename": frame.f_code.co_filename,
+                        "locals": {
+                            k: repr(v) for k, v in frame.f_locals.items()
+                        },
+                        "globals": {
+                            k: repr(v) for k, v in frame.f_globals.items()
+                        },
+                        "builtins": {
+                            k: repr(v) for k, v in frame.f_builtins.items()
+                        },
+                        "restricted": getattr(frame, "f_restricted", ""),
+                        "lasti": repr(frame.f_lasti),
+                        "exc_type": repr(getattr(frame, "f_exc_type", "")),
+                        "exc_value": repr(getattr(frame, "f_exc_value", "")),
+                        "exc_traceback": repr(
+                            getattr(frame, "f_exc_traceback", "")
+                        ),
+                        "current": frame is self.curframe,
+                    },
                 )
                 stack_data.append(frame_data)
-        self.output('stack', stack=stack_data)
+        self.output("stack", stack=stack_data)
 
     def reset(self):
         """
@@ -193,9 +201,9 @@ class Debugger(bdb.Bdb):
         while True:
             try:
                 command, args = self.commands.get(block=True)
-                if hasattr(self, 'do_{}'.format(command)):
+                if hasattr(self, "do_{}".format(command)):
                     try:
-                        resume = getattr(self, 'do_{}'.format(command))(**args)
+                        resume = getattr(self, "do_{}".format(command))(**args)
                         if resume:
                             # Resume running the program after the interaction.
                             break
@@ -203,33 +211,36 @@ class Debugger(bdb.Bdb):
                         raise
                     except Exception as ex:
                         msg = 'Unhandled error with command "{}": {}'.format(
-                            command, ex)
-                        self.output('error', message=msg)
+                            command, ex
+                        )
+                        self.output("error", message=msg)
                 else:
-                    self.output('error', message='Unknown command: {}'.format(
-                        command))
+                    self.output(
+                        "error", message="Unknown command: {}".format(command)
+                    )
             except (OSError, AttributeError, ClientClose):
                 # Connection problem; try listening for new connection.
                 client, addr = self.socket.accept()
                 self.client = client
                 self.commands = Queue()
-                self.command_thread = Thread(target=command_buffer,
-                                             args=(self, ))
+                self.command_thread = Thread(
+                    target=command_buffer, args=(self,)
+                )
                 self.command_thread.daemon = True
                 self.command_thread.start()
                 self.output(
-                    'bootstrap',
+                    "bootstrap",
                     breakpoints=[
                         {
-                            'bpnum': bp.number,
-                            'filename': bp.file,
-                            'line': bp.line,
-                            'temporary': bp.temporary,
-                            'enabled': bp.enabled,
-                            'funcname': bp.funcname
+                            "bpnum": bp.number,
+                            "filename": bp.file,
+                            "line": bp.line,
+                            "temporary": bp.temporary,
+                            "enabled": bp.enabled,
+                            "funcname": bp.funcname,
                         }
                         for bp in bdb.Breakpoint.bpbynumber[1:]
-                    ]
+                    ],
                 )
                 self.output_stack()
         # End
@@ -247,7 +258,7 @@ class Debugger(bdb.Bdb):
         if self._run_state == DebugState.STARTING:
             return
         if self.stop_here(frame):
-            self.output('call', args=argument_list)
+            self.output("call", args=argument_list)
             self.interact(frame, None)
 
     def user_line(self, frame):
@@ -259,8 +270,11 @@ class Debugger(bdb.Bdb):
         """
         if self._run_state == DebugState.STARTING:
             self._run_state = DebugState.STARTED
-        self.output('line', filename=self.canonic(frame.f_code.co_filename),
-                    line=frame.f_lineno)
+        self.output(
+            "line",
+            filename=self.canonic(frame.f_code.co_filename),
+            line=frame.f_lineno,
+        )
         self.interact(frame, None)
 
     def user_return(self, frame, return_value):
@@ -272,8 +286,8 @@ class Debugger(bdb.Bdb):
         """
         if self._run_state == DebugState.STARTING:
             return
-        frame.f_locals['__return__'] = return_value
-        self.output('return', retval=repr(return_value))
+        frame.f_locals["__return__"] = return_value
+        self.output("return", retval=repr(return_value))
         self.interact(frame, None)
 
     def user_exception(self, frame, exc_info):
@@ -287,12 +301,12 @@ class Debugger(bdb.Bdb):
         if self._run_state == DebugState.STARTING:
             return
         exc_type, exc_value, exc_traceback = exc_info
-        frame.f_locals['__exception__'] = exc_type, exc_value
+        frame.f_locals["__exception__"] = exc_type, exc_value
         if isinstance(exc_type, str):
             exc_type_name = exc_type
         else:
             exc_type_name = exc_type.__name__
-        self.output('exception', name=exc_type_name, value=repr(exc_value))
+        self.output("exception", name=exc_type_name, value=repr(exc_value))
         self.interact(frame, exc_traceback)
 
     # Debug command handlers.
@@ -301,25 +315,27 @@ class Debugger(bdb.Bdb):
         """
         Set a breakpoint.
         """
-        globs = self.curframe.f_globals if hasattr(self, 'curframe') else None
+        globs = self.curframe.f_globals if hasattr(self, "curframe") else None
         code = linecache.getline(filename, line, globs)
         if is_breakpoint_line(code):
             err = self.set_break(filename, line, temporary, None, None)
             if err:
-                self.output('error', message=err)
+                self.output("error", message=err)
             else:
                 bp = self.get_breaks(filename, line)[-1]
                 self.output(
-                    'breakpoint_create',
+                    "breakpoint_create",
                     bpnum=bp.number,
                     filename=bp.file,
                     line=bp.line,
                     temporary=bp.temporary,
-                    funcname=bp.funcname
+                    funcname=bp.funcname,
                 )
         else:
-            self.output('error', message='{}:{} is not executable'.format(
-                filename, line))
+            self.output(
+                "error",
+                message="{}:{} is not executable".format(filename, line),
+            )
 
     def do_enable(self, bpnum):
         """
@@ -327,12 +343,13 @@ class Debugger(bdb.Bdb):
         """
         bpnum = int(bpnum)
         if not (0 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
-            self.output('error',
-                        message='No breakpoint numbered {}'.format(bpnum))
+            self.output(
+                "error", message="No breakpoint numbered {}".format(bpnum)
+            )
         else:
             bp = bdb.Breakpoint.bpbynumber[bpnum]
             bp.enable()
-            self.output('breakpoint_enable', bpnum=bpnum)
+            self.output("breakpoint_enable", bpnum=bpnum)
 
     def do_disable(self, bpnum):
         """
@@ -340,12 +357,13 @@ class Debugger(bdb.Bdb):
         """
         bpnum = int(bpnum)
         if not (0 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
-            self.output('error',
-                        message='No breakpoint numbered {}'.format(bpnum))
+            self.output(
+                "error", message="No breakpoint numbered {}".format(bpnum)
+            )
         else:
             bp = bdb.Breakpoint.bpbynumber[bpnum]
             bp.disable()
-            self.output('breakpoint_disable', bpnum=bpnum)
+            self.output("breakpoint_disable", bpnum=bpnum)
 
     def do_ignore(self, bpnum, count):
         """
@@ -357,15 +375,16 @@ class Debugger(bdb.Bdb):
         except ValueError:
             count = 0
         if not (0 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
-            self.output('error',
-                        message='No breakpoint numbered {}'.format(bpnum))
+            self.output(
+                "error", message="No breakpoint numbered {}".format(bpnum)
+            )
         else:
             bp = bdb.Breakpoint.bpbynumber[bpnum]
             bp.ignore = count
             if count > 0:
-                self.output('breakpoint_ignore', bpnum=bpnum, count=count)
+                self.output("breakpoint_ignore", bpnum=bpnum, count=count)
             else:
-                self.output('breakpoint_enable', bpnum=bpnum)
+                self.output("breakpoint_enable", bpnum=bpnum)
 
     def do_clear(self, bpnum):
         """
@@ -373,13 +392,13 @@ class Debugger(bdb.Bdb):
         """
         bpnum = int(bpnum)
         if not (0 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
-            self.output('error', message='No breakpoint numbered %s' % bpnum)
+            self.output("error", message="No breakpoint numbered %s" % bpnum)
         else:
             err = self.clear_bpbynumber(bpnum)
             if err:
-                self.output('error', message=err)
+                self.output("error", message=err)
             else:
-                self.output('breakpoint_clear', bpnum=bpnum)
+                self.output("breakpoint_clear", bpnum=bpnum)
 
     def do_step(self):
         """
@@ -448,12 +467,15 @@ class Debugger(bdb.Bdb):
         """
         # Clean __main__
         import __main__
+
         __main__.__dict__.clear()
-        __main__.__dict__.update({
-            "__name__": "__main__",
-            "__file__": filename,
-            "__builtins__": __builtins__,
-        })
+        __main__.__dict__.update(
+            {
+                "__name__": "__main__",
+                "__file__": filename,
+                "__builtins__": __builtins__,
+            }
+        )
         # When bdb sets tracing, a number of call and line events happen BEFORE
         # we get to the user's code. The following measures avoid stopping
         # before we get to the target script (see user_line and user_call for
@@ -461,19 +483,22 @@ class Debugger(bdb.Bdb):
         self._run_state = DebugState.STARTING
         self.mainpyfile = self.canonic(filename)
         self._user_requested_quit = True  # End the process once completed.
-        e = ('__debug_script__ = open(r"{filename}", "rb");'
-             '__debug_code__ = compile(__debug_script__.read(),'
-             ' r"{filename}", "exec");'
-             'exec(__debug_code__);'
-             '__debug_script__.close();'.format(filename=filename))
+        e = (
+            '__debug_script__ = open(r"{filename}", "rb");'
+            "__debug_code__ = compile(__debug_script__.read(),"
+            ' r"{filename}", "exec");'
+            "exec(__debug_code__);"
+            "__debug_script__.close();".format(filename=filename)
+        )
         self.run(e)
 
 
-def run(hostname, port, filename, *args):
+def run(hostname, port, filename, args):
     """
     Run a Python script identified by "filename" with the specified arguments
     in a debugger session that's listening at hostname/port.
     """
+    logger.debug("runner.run %s:%s %s %r", hostname, port, filename, args)
     # Create the correct context for the target Python script.
     sys.argv[0] = filename
     sys.argv[1:] = args
@@ -489,21 +514,27 @@ def run(hostname, port, filename, *args):
     debugger = Debugger(s, hostname, port)
     debugger.reset()
 
+    print(
+        "Running in debug mode. Use the Stop, Continue, and Step toolbar"
+        " buttons to debug the script",
+        file=sys.stderr,
+    )
+
     while True:
         try:
             debugger._runscript(filename)
             if debugger._user_requested_quit:
-                debugger.output('finished')
+                debugger.output("finished")
                 break
         except Restart:
             # TODO: Log restart.
-            debugger.output('restart')
+            debugger.output("restart")
         except (KeyboardInterrupt, SystemExit, OSError):
             debugger.client = None
             break
         except Exception:
             msg = traceback.format_exc()
-            debugger.output('postmortem', exception=msg)
+            debugger.output("postmortem", exception=msg)
             debugger.client = None
             break
     # Close connection in a tidy manner.
