@@ -1,4 +1,4 @@
 from .main import Window
 
 
-__all__ = ['Window', ]
+__all__ = ["Window"]
