@@ -17,11 +17,15 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 import logging
-import os.path
-from mu.modes.base import BaseMode
-from mu.logic import DEBUGGER_PORT
-from mu.debugger.client import Debugger
-from mu.debugger.utils import is_breakpoint_line
+import os
+
+from .base import BaseMode
+from ..debugger.config import DEBUGGER_PORT
+from ..debugger.client import Debugger
+from ..debugger.utils import is_breakpoint_line
+from ..virtual_environment import venv
+
+from PyQt5.QtCore import QTimer
 
 
 logger = logging.getLogger(__name__)
@@ -32,13 +36,20 @@ class DebugMode(BaseMode):
     Represents the functionality required by the Python 3 visual debugger.
     """
 
-    name = _('Graphical Debugger')
-    description = _('Debug your Python 3 code.')
-    icon = 'python'
+    name = _("Graphical Debugger")
+    short_name = "debugger"
+    description = _("Debug your Python 3 code.")
+    icon = "python"
     runner = None
     is_debugger = True
     save_timeout = 0  # No need to auto-save when in read-only debug mode.
 
+    def __init__(self, editor, view):
+        super().__init__(editor, view)
+        self._button_disable_timer = QTimer()
+        self._button_disable_timer.setSingleShot(True)
+        self._button_disable_timer.timeout.connect(self.disable_buttons)
+
     def actions(self):
         """
         Return an ordered list of actions provided by this module. An action
@@ -46,39 +57,39 @@ class DebugMode(BaseMode):
         """
         return [
             {
-                'name': 'stop',
-                'display_name': _('Stop'),
-                'description': _('Stop the running code.'),
-                'handler': self.button_stop,
-                'shortcut': 'Shift+F5',
+                "name": "stop",
+                "display_name": _("Stop"),
+                "description": _("Stop the running code."),
+                "handler": self.button_stop,
+                "shortcut": "Shift+F5",
             },
             {
-                'name': 'run',
-                'display_name': _('Continue'),
-                'description': _('Continue to run your Python script.'),
-                'handler': self.button_continue,
-                'shortcut': 'F5',
+                "name": "run",
+                "display_name": _("Continue"),
+                "description": _("Continue to run your Python script."),
+                "handler": self.button_continue,
+                "shortcut": "F5",
             },
             {
-                'name': 'step-over',
-                'display_name': _('Step Over'),
-                'description': _('Step over a line of code.'),
-                'handler': self.button_step_over,
-                'shortcut': 'F10',
+                "name": "step-over",
+                "display_name": _("Step Over"),
+                "description": _("Step over a line of code."),
+                "handler": self.button_step_over,
+                "shortcut": "F10",
             },
             {
-                'name': 'step-in',
-                'display_name': _('Step In'),
-                'description': _('Step into a function.'),
-                'handler': self.button_step_in,
-                'shortcut': 'F11',
+                "name": "step-in",
+                "display_name": _("Step In"),
+                "description": _("Step into a function."),
+                "handler": self.button_step_in,
+                "shortcut": "F11",
             },
             {
-                'name': 'step-out',
-                'display_name': _('Step Out'),
-                'description': _('Step out of a function.'),
-                'handler': self.button_step_out,
-                'shortcut': 'Shift+F11',
+                "name": "step-out",
+                "display_name": _("Step Out"),
+                "description": _("Step out of a function."),
+                "handler": self.button_step_out,
+                "shortcut": "Shift+F11",
             },
         ]
 
@@ -96,7 +107,7 @@ class DebugMode(BaseMode):
         # Grab the Python file.
         tab = self.view.current_tab
         if tab is None:
-            logger.debug('There is no active text editor.')
+            logger.debug("There is no active text editor.")
             self.stop()
             return
         if tab.path is None:
@@ -110,51 +121,81 @@ class DebugMode(BaseMode):
             self.set_buttons(modes=False)
             envars = self.editor.envars
             cwd = os.path.dirname(tab.path)
-            self.runner = self.view.add_python3_runner(tab.path,
-                                                       cwd,
-                                                       debugger=True,
-                                                       envars=envars)
+            self.runner = self.view.add_python3_runner(
+                venv.interpreter, tab.path, cwd, debugger=True, envars=envars
+            )
             self.runner.process.waitForStarted()
             self.runner.process.finished.connect(self.finished)
             self.view.add_debug_inspector()
             self.view.set_read_only(True)
-            self.debugger = Debugger('localhost', DEBUGGER_PORT,
-                                     proc=self.runner.process)
+            self.debugger = Debugger(
+                "localhost", DEBUGGER_PORT, proc=self.runner.process
+            )
             self.debugger.view = self
             self.debugger.start()
         else:
-            logger.debug('Current script has not been saved. Aborting debug.')
+            logger.debug("Current script has not been saved. Aborting debug.")
             self.stop()
 
     def stop(self):
         """
         Stop the debug runner and reset the UI.
         """
-        logger.debug('Stopping debugger.')
+        logger.debug("Stopping debugger.")
         if self.runner:
-            self.runner.process.kill()
-            self.runner.process.waitForFinished()
+            self.runner.stop_process()
             self.runner = None
             self.debugger = None
             self.view.remove_python_runner()
             self.view.remove_debug_inspector()
         self.set_buttons(modes=True)
-        self.editor.change_mode('python')
-        self.editor.mode = 'python'
+        self.editor.change_mode("python")
+        self.editor.mode = "python"
         self.view.set_read_only(False)
 
     def finished(self):
         """
         Called when the debugged Python process is finished.
         """
-        buttons = {action['name']: False for action in self.actions()
-                   if action['name'] != 'stop'}
-        self.set_buttons(**buttons)
+        self.disable_buttons()
         self.editor.show_status_message(_("Your script has finished running."))
         for tab in self.view.widgets:
             tab.setSelection(0, 0, 0, 0)
             tab.reset_debugger_highlight()
 
+    def _enable_buttons(self, *, enable):
+        """
+        Enable/disable all debug control buttons except 'stop'.
+        """
+        buttons = {
+            action["name"]: enable
+            for action in self.actions()
+            if action["name"] != "stop"
+        }
+        self.set_buttons(**buttons)
+
+    def disable_buttons(self):
+        """
+        Disable all debug control buttons except 'stop'.
+        """
+        self._enable_buttons(enable=False)
+
+    def disable_buttons_later(self, *, milliseconds=100):
+        """
+        Set a timer to disable all debug control buttons except 'stop'.
+        """
+        self._button_disable_timer.start(milliseconds)
+
+    def enable_buttons(self):
+        """
+        Enable all debug control buttons except 'stop': if the timer started
+        in `disable_buttons_later` is active, stops it and does nothing else.
+        """
+        if self._button_disable_timer.isActive():
+            self._button_disable_timer.stop()
+            return
+        self._enable_buttons(enable=True)
+
     def button_stop(self, event):
         """
         Button clicked to stop the current script and return to Python3 mode.
@@ -165,6 +206,7 @@ class DebugMode(BaseMode):
         """
         Button clicked to continue running the script.
         """
+        self.disable_buttons_later()
         self.view.current_tab.reset_debugger_highlight()
         self.debugger.do_run()
 
@@ -172,6 +214,7 @@ class DebugMode(BaseMode):
         """
         Button clicked to step over the current line of code.
         """
+        self.disable_buttons_later()
         self.view.current_tab.reset_debugger_highlight()
         self.debugger.do_next()
 
@@ -179,6 +222,7 @@ class DebugMode(BaseMode):
         """
         Button clicked to step into the current block of code.
         """
+        self.disable_buttons_later()
         self.view.current_tab.reset_debugger_highlight()
         self.debugger.do_step()
 
@@ -186,6 +230,7 @@ class DebugMode(BaseMode):
         """
         Button clicked to step out of the current block of code.
         """
+        self.disable_buttons_later()
         self.view.current_tab.reset_debugger_highlight()
         self.debugger.do_return()
 
@@ -219,7 +264,7 @@ class DebugMode(BaseMode):
         process_runner = self.view.process_runner
         if process_runner:
             msg = _("Unable to connect to the Python debugger.\n\n") + message
-            process_runner.append(msg.encode('utf-8'))
+            process_runner.append(msg.encode("utf-8"))
             # Set the state to finished.
             self.finished()
             process_runner.finished(1, -1)
@@ -238,8 +283,11 @@ class DebugMode(BaseMode):
             for handle in list(tab.breakpoint_handles):
                 line = tab.markerLine(handle)
                 code = tab.text(line)
-                if line > -1 and line not in break_lines and \
-                        is_breakpoint_line(code):
+                if (
+                    line > -1
+                    and line not in break_lines
+                    and is_breakpoint_line(code)
+                ):
                     self.debugger.create_breakpoint(tab.path, line + 1)
                     break_lines.add(line)
                 else:
@@ -253,8 +301,9 @@ class DebugMode(BaseMode):
         Handle when a breakpoint is enabled.
         """
         tab = self.view.current_tab
-        if tab.path == breakpoint.filename and \
-                not tab.markersAtLine(breakpoint.line - 1):
+        if tab.path == breakpoint.filename and not tab.markersAtLine(
+            breakpoint.line - 1
+        ):
             tab.markerAdd(breakpoint.line - 1, tab.BREAKPOINT_MARKER)
 
     def debug_on_breakpoint_disable(self, breakpoint):
@@ -268,7 +317,8 @@ class DebugMode(BaseMode):
         """
         Handle when the debugger has moved to the referenced line in the file.
         """
-        ignored = ['bdb.py', ]  # Files the debugger should ignore.
+        self.enable_buttons()
+        ignored = ["bdb.py"]  # Files the debugger should ignore.
         if os.path.basename(filename) in ignored:
             self.debugger.do_return()
             return
@@ -283,7 +333,7 @@ class DebugMode(BaseMode):
         if stack:
             locals_dict = {}
             for frame in stack:
-                for k, v in frame[1]['locals'].items():
+                for k, v in frame[1]["locals"].items():
                     locals_dict[k] = v
             self.view.update_debug_inspector(locals_dict)
 
@@ -293,10 +343,10 @@ class DebugMode(BaseMode):
         """
         process_runner = self.view.process_runner
         for item in args:
-            process_runner.append(item.encode('utf-8'))
+            process_runner.append(item.encode("utf-8"))
         for k, v in kwargs.items():
-            msg = '{}: {}'.format(k, v)
-            process_runner.append(msg.encode('utf-8'))
+            msg = "{}: {}".format(k, v)
+            process_runner.append(msg.encode("utf-8"))
 
     def debug_on_info(self, message):
         """
@@ -308,15 +358,17 @@ class DebugMode(BaseMode):
         """
         Handle when the debugger sends a warning message.
         """
-        self.editor.show_status_message(_("Debugger warning: {}").format(
-            message))
+        self.editor.show_status_message(
+            _("Debugger warning: {}").format(message)
+        )
 
     def debug_on_error(self, message):
         """
         Handle when the debugger sends an error message.
         """
-        self.editor.show_status_message(_("Debugger error: {}").format(
-            message))
+        self.editor.show_status_message(
+            _("Debugger error: {}").format(message)
+        )
 
     def debug_on_call(self, args):
         """
