@@ -2,8 +2,9 @@
 """
 Tests for the debug mode.
 """
-from mu.logic import DEBUGGER_PORT
+from mu.debugger.config import DEBUGGER_PORT
 from mu.modes.debugger import DebugMode
+from mu.virtual_environment import venv
 from unittest import mock
 
 
@@ -14,9 +15,9 @@ def test_debug_mode():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
-    assert dm.name == 'Graphical Debugger'
+    assert dm.name == "Graphical Debugger"
     assert dm.description is not None
-    assert dm.icon == 'python'
+    assert dm.icon == "python"
     assert dm.runner is None
     assert dm.is_debugger is True
     assert dm.editor == editor
@@ -26,16 +27,16 @@ def test_debug_mode():
 
     actions = dm.actions()
     assert len(actions) == 5
-    assert actions[0]['name'] == 'stop'
-    assert actions[0]['handler'] == dm.button_stop
-    assert actions[1]['name'] == 'run'
-    assert actions[1]['handler'] == dm.button_continue
-    assert actions[2]['name'] == 'step-over'
-    assert actions[2]['handler'] == dm.button_step_over
-    assert actions[3]['name'] == 'step-in'
-    assert actions[3]['handler'] == dm.button_step_in
-    assert actions[4]['name'] == 'step-out'
-    assert actions[4]['handler'] == dm.button_step_out
+    assert actions[0]["name"] == "stop"
+    assert actions[0]["handler"] == dm.button_stop
+    assert actions[1]["name"] == "run"
+    assert actions[1]["handler"] == dm.button_continue
+    assert actions[2]["name"] == "step-over"
+    assert actions[2]["handler"] == dm.button_step_over
+    assert actions[3]["name"] == "step-in"
+    assert actions[3]["handler"] == dm.button_step_in
+    assert actions[4]["name"] == "step-out"
+    assert actions[4]["handler"] == dm.button_step_out
 
 
 def test_debug_start():
@@ -43,7 +44,7 @@ def test_debug_start():
     Ensure the handling of starting the debugger works as expected.
     """
     editor = mock.MagicMock()
-    editor.envars = [['name', 'value'], ]
+    editor.envars = [["name", "value"]]
     view = mock.MagicMock()
     view.current_tab.path = "/foo/bar"
     view.current_tab.isModified.return_value = True
@@ -52,19 +53,25 @@ def test_debug_start():
     mock_debugger = mock.MagicMock()
     mock_debugger_class = mock.MagicMock(return_value=mock_debugger)
     dm = DebugMode(editor, view)
-    dm.workspace_dir = mock.MagicMock(return_value='/bar')
-    with mock.patch('mu.modes.debugger.Debugger', mock_debugger_class):
+    with mock.patch(
+        "mu.modes.debugger.Debugger", mock_debugger_class
+    ), mock.patch.object(venv, "interpreter", "interpreter"):
         dm.start()
     editor.save_tab_to_file.called_once_with(view.current_tab)
-    view.add_python3_runner.assert_called_once_with('/foo/bar', '/foo',
-                                                    debugger=True,
-                                                    envars=[['name', 'value']])
+    view.add_python3_runner.assert_called_once_with(
+        "interpreter",
+        "/foo/bar",
+        "/foo",
+        debugger=True,
+        envars=[["name", "value"]],
+    )
     mock_runner.process.waitForStarted.assert_called_once_with()
     mock_runner.process.finished.connect.assert_called_once_with(dm.finished)
     view.add_debug_inspector.assert_called_once_with()
     view.set_read_only.assert_called_once_with(True)
-    mock_debugger_class.assert_called_once_with('localhost', DEBUGGER_PORT,
-                                                proc=mock_runner.process)
+    mock_debugger_class.assert_called_once_with(
+        "localhost", DEBUGGER_PORT, proc=mock_runner.process
+    )
     assert dm.runner == mock_runner
     assert dm.debugger == mock_debugger
     assert mock_debugger.view == dm
@@ -111,12 +118,11 @@ def test_debug_stop():
     dm.stop()
     assert dm.runner is None
     assert dm.debugger is None
-    mock_runner.process.kill.assert_called_once_with()
-    mock_runner.process.waitForFinished.assert_called_once_with()
+    mock_runner.stop_process.assert_called_once_with()
     view.remove_python_runner.assert_called_once_with()
     view.remove_debug_inspector.assert_called_once_with()
-    editor.change_mode.assert_called_once_with('python')
-    assert editor.mode == 'python'
+    editor.change_mode.assert_called_once_with("python")
+    assert editor.mode == "python"
     view.set_read_only.assert_called_once_with(False)
 
 
@@ -128,11 +134,11 @@ def test_debug_finished():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     view.button_bar.slots = {
-        'stop': mock.MagicMock(),
-        'run': mock.MagicMock(),
-        'step-over': mock.MagicMock(),
-        'step-in': mock.MagicMock(),
-        'step-out': mock.MagicMock(),
+        "stop": mock.MagicMock(),
+        "run": mock.MagicMock(),
+        "step-over": mock.MagicMock(),
+        "step-in": mock.MagicMock(),
+        "step-out": mock.MagicMock(),
     }
     dm = DebugMode(editor, view)
     mock_debugger = mock.MagicMock()
@@ -140,24 +146,20 @@ def test_debug_finished():
     mock_debugger.bp_index = []
     mock_breakpoint = mock.MagicMock()
     mock_breakpoint.enabled = True
-    mock_debugger.breakpoints.side_effect = [
-        {
-            1: mock_breakpoint,
-        },
-        {},
-    ]
+    mock_debugger.breakpoints.side_effect = [{1: mock_breakpoint}, {}]
     tab1 = mock.MagicMock()
-    tab1.path = 'foo'
+    tab1.path = "foo"
     tab2 = mock.MagicMock()
     view.widgets = [tab1, tab2]
     dm.finished()
     # Buttons are set to the right state.
-    assert view.button_bar.slots['stop'].setEnabled.call_count == 0
-    view.button_bar.slots['run'].setEnabled.assert_called_once_with(False)
-    view.button_bar.slots['step-over'].\
-        setEnabled.assert_called_once_with(False)
-    view.button_bar.slots['step-in'].setEnabled.assert_called_once_with(False)
-    view.button_bar.slots['step-out'].setEnabled.assert_called_once_with(False)
+    assert view.button_bar.slots["stop"].setEnabled.call_count == 0
+    view.button_bar.slots["run"].setEnabled.assert_called_once_with(False)
+    view.button_bar.slots["step-over"].setEnabled.assert_called_once_with(
+        False
+    )
+    view.button_bar.slots["step-in"].setEnabled.assert_called_once_with(False)
+    view.button_bar.slots["step-out"].setEnabled.assert_called_once_with(False)
     # Tabs are set to the right state.
     tab1.setSelection.assert_called_once_with(0, 0, 0, 0)
     tab1.reset_debugger_highlight.assert_called_once_with()
@@ -239,20 +241,17 @@ def test_debug_toggle_breakpoint_off():
     mock_debugger = mock.MagicMock()
     dm.debugger = mock_debugger
     mock_breakpoint = mock.MagicMock()
-    mock_debugger.breakpoints.side_effect = [
-        {
-            1: mock_breakpoint,
-        },
-    ]
+    mock_debugger.breakpoints.side_effect = [{1: mock_breakpoint}]
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.markersAtLine.return_value = True
     dm.toggle_breakpoint(0, mock_tab)
     mock_debugger.breakpoints.assert_called_once_with(mock_tab.path)
     mock_tab.markersAtLine.assert_called_once_with(0)
     mock_debugger.disable_breakpoint.assert_called_once_with(mock_breakpoint)
-    mock_tab.markerDelete.assert_called_once_with(0,
-                                                  mock_tab.BREAKPOINT_MARKER)
+    mock_tab.markerDelete.assert_called_once_with(
+        0, mock_tab.BREAKPOINT_MARKER
+    )
 
 
 def test_debug_toggle_breakpoint_off_no_breakpoint():
@@ -267,18 +266,17 @@ def test_debug_toggle_breakpoint_off_no_breakpoint():
     dm = DebugMode(editor, view)
     mock_debugger = mock.MagicMock()
     dm.debugger = mock_debugger
-    mock_debugger.breakpoints.side_effect = [
-        {}
-    ]
+    mock_debugger.breakpoints.side_effect = [{}]
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.markersAtLine.return_value = True
     dm.toggle_breakpoint(0, mock_tab)
     mock_debugger.breakpoints.assert_called_once_with(mock_tab.path)
     mock_tab.markersAtLine.assert_called_once_with(0)
     assert mock_debugger.disable_breakpoint.call_count == 0
-    mock_tab.markerDelete.assert_called_once_with(0,
-                                                  mock_tab.BREAKPOINT_MARKER)
+    mock_tab.markerDelete.assert_called_once_with(
+        0, mock_tab.BREAKPOINT_MARKER
+    )
 
 
 def test_debug_toggle_breakpoint_on_new():
@@ -291,13 +289,9 @@ def test_debug_toggle_breakpoint_on_new():
     mock_debugger = mock.MagicMock()
     dm.debugger = mock_debugger
     mock_breakpoint = mock.MagicMock()
-    mock_debugger.breakpoints.side_effect = [
-        {
-            1: mock_breakpoint,
-        },
-    ]
+    mock_debugger.breakpoints.side_effect = [{1: mock_breakpoint}]
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.markersAtLine.return_value = False
     dm.toggle_breakpoint(0, mock_tab)
     dm.debugger.enable_breakpoint.assert_called_once_with(mock_breakpoint)
@@ -314,7 +308,7 @@ def test_debug_toggle_breakpoint_on_existing():
     dm.debugger = mock_debugger
     mock_debugger.breakpoints.return_value = {}
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.markersAtLine.return_value = False
     dm.toggle_breakpoint(0, mock_tab)
     dm.debugger.create_breakpoint.assert_called_once_with(mock_tab.path, 1)
@@ -330,7 +324,7 @@ def test_debug_on_fail():
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
     dm.finished = mock.MagicMock()
-    dm.debug_on_fail('This is a useful message')
+    dm.debug_on_fail("This is a useful message")
     assert view.process_runner.append.call_count == 1  # message shown.
     dm.finished.assert_called_once_with()
     view.process_runner.finished.assert_called_once_with(1, -1)
@@ -345,11 +339,11 @@ def test_debug_on_bootstrap():
     dm = DebugMode(editor, view)
     dm.debugger = mock.MagicMock()
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.text.return_value = "print('Hello')"
-    mock_tab.breakpoint_handles = set([0, ])
+    mock_tab.breakpoint_handles = set([0])
     mock_tab.markerLine.return_value = 0
-    view.widgets = [mock_tab, ]
+    view.widgets = [mock_tab]
     dm.debug_on_bootstrap()
     dm.debugger.create_breakpoint.assert_called_once_with(mock_tab.path, 1)
     dm.debugger.do_run.assert_called_once_with()
@@ -365,10 +359,10 @@ def test_debug_on_bootstrap_remove_missing_marker_handles():
     dm = DebugMode(editor, view)
     dm.debugger = mock.MagicMock()
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
-    mock_tab.breakpoint_handles = set([0, ])
+    mock_tab.path = "foo"
+    mock_tab.breakpoint_handles = set([0])
     mock_tab.markerLine.return_value = -1
-    view.widgets = [mock_tab, ]
+    view.widgets = [mock_tab]
     dm.debug_on_bootstrap()
     assert dm.debugger.create_breakpoint.call_count == 0
     assert 0 not in mock_tab.breakpoint_handles
@@ -385,11 +379,11 @@ def test_debug_on_bootstrap_ignore_duplicate_handles():
     dm = DebugMode(editor, view)
     dm.debugger = mock.MagicMock()
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     mock_tab.text.return_value = "print('Hello')"
     mock_tab.breakpoint_handles = set([0, 1])
     mock_tab.markerLine.side_effect = [1, 1]
-    view.widgets = [mock_tab, ]
+    view.widgets = [mock_tab]
     dm.debug_on_bootstrap()
     assert dm.debugger.create_breakpoint.call_count == 1
 
@@ -406,12 +400,13 @@ def test_debug_on_bootstrap_remove_invalid_breaks():
     dm = DebugMode(editor, view)
     dm.debugger = mock.MagicMock()
     mock_tab = mock.MagicMock()
-    mock_tab.path = 'foo'
-    mock_tab.breakpoint_handles = set([0, ])
+    mock_tab.path = "foo"
+    mock_tab.breakpoint_handles = set([0])
     mock_tab.markerLine.return_value = 1
-    view.widgets = [mock_tab, ]
-    with mock.patch('mu.modes.debugger.is_breakpoint_line',
-                    return_value=False):
+    view.widgets = [mock_tab]
+    with mock.patch(
+        "mu.modes.debugger.is_breakpoint_line", return_value=False
+    ):
         dm.debug_on_bootstrap()
     assert 0 not in mock_tab.breakpoint_handles
     mock_tab.markerDelete.assert_called_once_with(1, -1)
@@ -425,15 +420,16 @@ def test_debug_on_breakpoint_enable():
     view = mock.MagicMock()
     mock_tab = mock.MagicMock()
     mock_tab.markersAtLine.return_value = False
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     view.current_tab = mock_tab
     dm = DebugMode(editor, view)
     mock_breakpoint = mock.MagicMock()
-    mock_breakpoint.filename = 'foo'
+    mock_breakpoint.filename = "foo"
     mock_breakpoint.line = 1
     dm.debug_on_breakpoint_enable(mock_breakpoint)
-    mock_tab.markerAdd.assert_called_once_with(mock_breakpoint.line - 1,
-                                               mock_tab.BREAKPOINT_MARKER)
+    mock_tab.markerAdd.assert_called_once_with(
+        mock_breakpoint.line - 1, mock_tab.BREAKPOINT_MARKER
+    )
 
 
 def test_debug_on_breakpoint_enable_different_tab():
@@ -448,11 +444,11 @@ def test_debug_on_breakpoint_enable_different_tab():
     view = mock.MagicMock()
     mock_tab = mock.MagicMock()
     mock_tab.markersAtLine.return_value = False
-    mock_tab.path = 'foo'
+    mock_tab.path = "foo"
     view.current_tab = mock_tab
     dm = DebugMode(editor, view)
     mock_breakpoint = mock.MagicMock()
-    mock_breakpoint.filename = 'bar'
+    mock_breakpoint.filename = "bar"
     mock_breakpoint.line = 1
     dm.debug_on_breakpoint_enable(mock_breakpoint)
     assert mock_tab.markerAdd.call_count == 0
@@ -489,8 +485,9 @@ def test_debug_on_breakpoint_disable():
     mock_breakpoint = mock.MagicMock()
     mock_breakpoint.line = 1
     dm.debug_on_breakpoint_disable(mock_breakpoint)
-    mock_tab.markerDelete.assert_called_once_with(mock_breakpoint.line - 1,
-                                                  mock_tab.BREAKPOINT_MARKER)
+    mock_tab.markerDelete.assert_called_once_with(
+        mock_breakpoint.line - 1, mock_tab.BREAKPOINT_MARKER
+    )
 
 
 def test_debug_on_line_ignore_file():
@@ -501,7 +498,7 @@ def test_debug_on_line_ignore_file():
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
     dm.debugger = mock.MagicMock()
-    dm.debug_on_line('bdb.py', 100)
+    dm.debug_on_line("bdb.py", 100)
     dm.debugger.do_return.assert_called_once_with()
 
 
@@ -515,7 +512,7 @@ def test_debug_on_line():
     dm = DebugMode(editor, view)
     mock_tab = mock.MagicMock()
     editor.get_tab.return_value = mock_tab
-    dm.debug_on_line('foo.py', 100)
+    dm.debug_on_line("foo.py", 100)
     view.current_tab.setSelection.assert_called_once_with(0, 0, 0, 0)
     mock_tab.setSelection(99, 0, 100, 0)
 
@@ -542,31 +539,13 @@ def test_debug_on_stack():
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
     stack = [
-        (
-            1,
-            {
-                'locals': {
-                    'a': 'frame1',
-                    'b': 'frame1',
-                }
-            }
-        ),
-        (
-            2,
-            {
-                'locals': {
-                    'b': 'frame2',
-                    'c': 'frame2',
-                }
-            }
-        )
+        (1, {"locals": {"a": "frame1", "b": "frame1"}}),
+        (2, {"locals": {"b": "frame2", "c": "frame2"}}),
     ]
     dm.debug_on_stack(stack)
-    view.update_debug_inspector.assert_called_once_with({
-        'a': 'frame1',
-        'b': 'frame2',
-        'c': 'frame2',
-    })
+    view.update_debug_inspector.assert_called_once_with(
+        {"a": "frame1", "b": "frame2", "c": "frame2"}
+    )
 
 
 def test_debug_on_postmortem():
@@ -577,10 +556,8 @@ def test_debug_on_postmortem():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
-    args = ['foo', 'bar']
-    kwargs = {
-        'baz': 'qux',
-    }
+    args = ["foo", "bar"]
+    kwargs = {"baz": "qux"}
     dm.debug_on_postmortem(args, kwargs)
     assert view.process_runner.append.call_count == 3
 
@@ -592,8 +569,8 @@ def test_debug_on_info():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
-    dm.debug_on_info('message')
-    expected = 'Debugger info: message'
+    dm.debug_on_info("message")
+    expected = "Debugger info: message"
     editor.show_status_message.assert_called_once_with(expected)
 
 
@@ -604,8 +581,8 @@ def test_debug_on_warning():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
-    dm.debug_on_warning('message')
-    expected = 'Debugger warning: message'
+    dm.debug_on_warning("message")
+    expected = "Debugger warning: message"
     editor.show_status_message.assert_called_once_with(expected)
 
 
@@ -616,8 +593,8 @@ def test_debug_on_error():
     editor = mock.MagicMock()
     view = mock.MagicMock()
     dm = DebugMode(editor, view)
-    dm.debug_on_error('message')
-    expected = 'Debugger error: message'
+    dm.debug_on_error("message")
+    expected = "Debugger error: message"
     editor.show_status_message.assert_called_once_with(expected)
 
 
@@ -699,3 +676,31 @@ def test_debug_on_exception():
     dm.debug_on_exception("Exception", "Exception information")
     dm.debugger.do_run.assert_called_once_with()
     assert view.current_tab.reset_debugger_highlight.call_count == 1
+
+
+def test_enable_buttons_before_delayed_disable_buttons():
+    """
+    Calling `enable_buttons` before the disable button timer fires i) cancels
+    that timer and, ii) does not call the code that actually enables buttons.
+    """
+    editor = mock.MagicMock()
+    view = mock.MagicMock()
+    dm = DebugMode(editor, view)
+
+    # Mocked active timer.
+    timer_mock = mock.MagicMock()
+    timer_mock.isActive.return_value = True
+    dm._button_disable_timer = timer_mock
+
+    # Mocked underlying _enable_buttons method.
+    real_enable_method_mock = mock.MagicMock()
+    dm._enable_buttons = real_enable_method_mock
+
+    # Call enable_buttons to check behaviour.
+    dm.enable_buttons()
+
+    # Timer should have been stopped.
+    timer_mock.stop.assert_called_once_with()
+
+    # Underlying _enable_buttons method should not have been called.
+    assert real_enable_method_mock.call_count == 0
